/* -*- mode: c; coding: utf-8 -*- */

#include <IceC/IceC.h>

typedef struct {
    Ice_String key;
    Ice_String value;
} IDM_Context_Pair;

typedef IDM_Context_Pair* IDM_Context_PairPtr;

typedef struct {
    byte size;
    IDM_Context_PairPtr items;
} IDM_Context;

typedef IDM_Context* IDM_ContextPtr;

#define IDM_Context_init Ice_Dict_init

#define IDM_Context_set(DICT, INDEX, KEY, VALUE)        \
do {                                                    \
    Ice_String_init(DICT.items[INDEX].key, KEY);        \
    Ice_String_init(DICT.items[INDEX].value, VALUE);    \
} while(0)                                              

static void
IDM_Context_writeToOutputStream(IDM_ContextPtr self,
                                Ice_OutputStreamPtr os) {
    byte i = 0;

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeSize(os, self->size);
    
    for (i=0; i<self->size; i++) {
        Ice_OutputStream_writeIceString(os, self->items[i].key);
        Ice_OutputStream_writeIceString(os, self->items[i].value);
    }
}

static void
IDM_Context_readFromInputStream(IDM_ContextPtr self,
                                Ice_InputStreamPtr is) {
    Ice_Int size = 0;
    byte i = 0;

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readSize(is, &size, true);
    assert(size < 256);
    self->size = size;
    
    for (i=0; i<self->size; i++) {
        Ice_InputStream_readIceString(is, &(self->items[i].key));
        Ice_InputStream_readIceString(is, &(self->items[i].value));
    }
}

typedef struct {
    Ice_Int size;
    Ice_Byte* items;
} IDM_byteSeq;

typedef IDM_byteSeq* IDM_byteSeqPtr;

static void
IDM_byteSeq_writeToOutputStream(IDM_byteSeqPtr self,
                                Ice_OutputStreamPtr os) {
    int i = 0;

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeSize(os, self->size);
    
    for (i=0; i<self->size; i++) {
        Ice_OutputStream_writeByte(os, *(self->items + i));
    }
}

static void
IDM_byteSeq_readFromInputStream(IDM_byteSeqPtr self,
                                Ice_InputStreamPtr is) {

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readSize(is, &(self->size), true);
    
    /* zero-copy optimization */
    self->items = is->next;
    is->next += self->size;
}

typedef struct {
    Ice_Object __base;
} IDM_NeighborDiscovery_Listener;

typedef IDM_NeighborDiscovery_Listener* IDM_NeighborDiscovery_ListenerPtr;

typedef Ice_ObjectPrx IDM_NeighborDiscovery_ListenerPrx;
typedef Ice_ObjectPrx* IDM_NeighborDiscovery_ListenerPrxPtr;

/* forward declaration */
static void
IDM_NeighborDiscovery_Listener_methodHandler(IDM_NeighborDiscovery_ListenerPtr self,
                                             Ice_InputStreamPtr is,
                                             Ice_OutputStreamPtr os);

static void
IDM_NeighborDiscovery_Listener_init(IDM_NeighborDiscovery_ListenerPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)IDM_NeighborDiscovery_Listener_methodHandler);
}

void
IDM_NeighborDiscovery_ListenerI_adv(IDM_NeighborDiscovery_ListenerPtr self,
                                    Ice_String proxy) __attribute__((weak));

void
IDM_NeighborDiscovery_ListenerI_bye(IDM_NeighborDiscovery_ListenerPtr self,
                                    Ice_String oid) __attribute__((weak));

static void
IDM_NeighborDiscovery_Listener_adv__dispatch(IDM_NeighborDiscovery_ListenerPtr self,
                                             Ice_InputStreamPtr is,
                                             Ice_OutputStreamPtr os) {
    Ice_String proxy;

    trace();
    
    Ice_InputStream_readIceString(is, &proxy);
    
    IDM_NeighborDiscovery_ListenerI_adv(self, proxy);
}

static void
IDM_NeighborDiscovery_Listener_bye__dispatch(IDM_NeighborDiscovery_ListenerPtr self,
                                             Ice_InputStreamPtr is,
                                             Ice_OutputStreamPtr os) {
    Ice_String oid;

    trace();
    
    Ice_InputStream_readIceString(is, &oid);
    
    IDM_NeighborDiscovery_ListenerI_bye(self, oid);
}

static void
IDM_NeighborDiscovery_Listener_methodHandler(IDM_NeighborDiscovery_ListenerPtr self,
                                             Ice_InputStreamPtr is,
                                             Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "adv") == 0) {
        IDM_NeighborDiscovery_Listener_adv__dispatch(self, is, os);
        return;
    }
    
    if (strcmp(operation, "bye") == 0) {
        IDM_NeighborDiscovery_Listener_bye__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
IDM_NeighborDiscovery_Listener_adv(IDM_NeighborDiscovery_ListenerPrxPtr self,
                                   Ice_String proxy) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "adv");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeIceString(os, proxy);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

static void
IDM_NeighborDiscovery_Listener_bye(IDM_NeighborDiscovery_ListenerPrxPtr self,
                                   Ice_String oid) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "bye");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeIceString(os, oid);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_String destination;
    Ice_String operation;
    IDM_byteSeq inParams;
    IDM_Context theContext;
} IDM_Message;

typedef IDM_Message* IDM_MessagePtr;

static void
IDM_Message_writeToOutputStream(IDM_MessagePtr self,
                                Ice_OutputStreamPtr os) {

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeIceString(os, self->destination);
    Ice_OutputStream_writeIceString(os, self->operation);
    IDM_byteSeq_writeToOutputStream(&(self->inParams), os);
    IDM_Context_writeToOutputStream(&(self->theContext), os);
}

static void
IDM_Message_readFromInputStream(IDM_MessagePtr self,
                                Ice_InputStreamPtr is) {

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readIceString(is, &(self->destination));
    Ice_InputStream_readIceString(is, &(self->operation));
    IDM_byteSeq_readFromInputStream(&(self->inParams), is);
    IDM_Context_readFromInputStream(&(self->theContext), is);
}

typedef enum {
    ActionType_Forward
} IDM_Routing_ActionType;

typedef struct {
    Ice_String address;
} IDM_Routing_Matcher;

typedef IDM_Routing_Matcher* IDM_Routing_MatcherPtr;

static void
IDM_Routing_Matcher_writeToOutputStream(IDM_Routing_MatcherPtr self,
                                        Ice_OutputStreamPtr os) {

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeIceString(os, self->address);
}

static void
IDM_Routing_Matcher_readFromInputStream(IDM_Routing_MatcherPtr self,
                                        Ice_InputStreamPtr is) {

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readIceString(is, &(self->address));
}

typedef struct {
    IDM_Routing_ActionType action;
    Ice_String next;
} IDM_Routing_Action;

typedef IDM_Routing_Action* IDM_Routing_ActionPtr;

static void
IDM_Routing_Action_writeToOutputStream(IDM_Routing_ActionPtr self,
                                       Ice_OutputStreamPtr os) {

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeByte(os, self->action);
    Ice_OutputStream_writeIceString(os, self->next);
}

static void
IDM_Routing_Action_readFromInputStream(IDM_Routing_ActionPtr self,
                                       Ice_InputStreamPtr is) {

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readByte(is, (Ice_Byte*)&(self->action));
    Ice_InputStream_readIceString(is, &(self->next));
}

typedef struct {
    Ice_Int identifier;
    IDM_Routing_Matcher m;
    IDM_Routing_Action a;
} IDM_Routing_Flow;

typedef IDM_Routing_Flow* IDM_Routing_FlowPtr;

static void
IDM_Routing_Flow_writeToOutputStream(IDM_Routing_FlowPtr self,
                                     Ice_OutputStreamPtr os) {

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeInt(os, self->identifier);
    IDM_Routing_Matcher_writeToOutputStream(&(self->m), os);
    IDM_Routing_Action_writeToOutputStream(&(self->a), os);
}

static void
IDM_Routing_Flow_readFromInputStream(IDM_Routing_FlowPtr self,
                                     Ice_InputStreamPtr is) {

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readInt(is, &(self->identifier));
    IDM_Routing_Matcher_readFromInputStream(&(self->m), is);
    IDM_Routing_Action_readFromInputStream(&(self->a), is);
}

typedef struct {
    Ice_Int size;
    IDM_Routing_Flow* items;
} IDM_Routing_FlowSeq;

typedef IDM_Routing_FlowSeq* IDM_Routing_FlowSeqPtr;

static void
IDM_Routing_FlowSeq_writeToOutputStream(IDM_Routing_FlowSeqPtr self,
                                        Ice_OutputStreamPtr os) {
    int i = 0;

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeSize(os, self->size);
    
    for (i=0; i<self->size; i++) {
        IDM_Routing_Flow_writeToOutputStream(&(*(self->items + i)), os);
    }
}

static void
IDM_Routing_FlowSeq_readFromInputStream(IDM_Routing_FlowSeqPtr self,
                                        Ice_InputStreamPtr is) {
    int i = 0;

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readSize(is, &(self->size), true);
    
    for (i=0; i<self->size; i++) {
        IDM_Routing_Flow_readFromInputStream(self->items + i, is);
    }
}

typedef struct {
    Ice_Object __base;
} IDM_Routing_RouterAdmin;

typedef IDM_Routing_RouterAdmin* IDM_Routing_RouterAdminPtr;

typedef Ice_ObjectPrx IDM_Routing_RouterAdminPrx;
typedef Ice_ObjectPrx* IDM_Routing_RouterAdminPrxPtr;

/* forward declaration */
static void
IDM_Routing_RouterAdmin_methodHandler(IDM_Routing_RouterAdminPtr self,
                                      Ice_InputStreamPtr is,
                                      Ice_OutputStreamPtr os);

static void
IDM_Routing_RouterAdmin_init(IDM_Routing_RouterAdminPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)IDM_Routing_RouterAdmin_methodHandler);
}

void
IDM_Routing_RouterAdminI_flowAdd(IDM_Routing_RouterAdminPtr self,
                                 IDM_Routing_Flow f) __attribute__((weak));

void
IDM_Routing_RouterAdminI_flowDelete(IDM_Routing_RouterAdminPtr self,
                                    Ice_Int identifier) __attribute__((weak));

IDM_Routing_FlowSeq
IDM_Routing_RouterAdminI_flowList(IDM_Routing_RouterAdminPtr self) __attribute__((weak));

static void
IDM_Routing_RouterAdmin_flowAdd__dispatch(IDM_Routing_RouterAdminPtr self,
                                          Ice_InputStreamPtr is,
                                          Ice_OutputStreamPtr os) {
    IDM_Routing_Flow f;

    trace();
    
    IDM_Routing_Flow_readFromInputStream(&f, is);
    
    IDM_Routing_RouterAdminI_flowAdd(self, f);
}

static void
IDM_Routing_RouterAdmin_flowDelete__dispatch(IDM_Routing_RouterAdminPtr self,
                                             Ice_InputStreamPtr is,
                                             Ice_OutputStreamPtr os) {
    Ice_Int identifier = 0;

    trace();
    
    Ice_InputStream_readInt(is, &identifier);
    
    IDM_Routing_RouterAdminI_flowDelete(self, identifier);
}

static void
IDM_Routing_RouterAdmin_flowList__dispatch(IDM_Routing_RouterAdminPtr self,
                                           Ice_InputStreamPtr is,
                                           Ice_OutputStreamPtr os) {
    IDM_Routing_FlowSeq _retval;

    trace();
    
    _retval = IDM_Routing_RouterAdminI_flowList(self);
    IDM_Routing_FlowSeq_writeToOutputStream(&(_retval), os);
}

static void
IDM_Routing_RouterAdmin_methodHandler(IDM_Routing_RouterAdminPtr self,
                                      Ice_InputStreamPtr is,
                                      Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "flowAdd") == 0) {
        IDM_Routing_RouterAdmin_flowAdd__dispatch(self, is, os);
        return;
    }
    
    if (strcmp(operation, "flowDelete") == 0) {
        IDM_Routing_RouterAdmin_flowDelete__dispatch(self, is, os);
        return;
    }
    
    if (strcmp(operation, "flowList") == 0) {
        IDM_Routing_RouterAdmin_flowList__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
IDM_Routing_RouterAdmin_flowAdd(IDM_Routing_RouterAdminPrxPtr self,
                                IDM_Routing_Flow f) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "flowAdd");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    IDM_Routing_Flow_writeToOutputStream(&(f), os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

static void
IDM_Routing_RouterAdmin_flowDelete(IDM_Routing_RouterAdminPrxPtr self,
                                   Ice_Int identifier) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "flowDelete");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeInt(os, identifier);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

static IDM_Routing_FlowSeq
IDM_Routing_RouterAdmin_flowList(IDM_Routing_RouterAdminPrxPtr self) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "flowList");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Object __base;
} IDM_Router;

typedef IDM_Router* IDM_RouterPtr;

typedef Ice_ObjectPrx IDM_RouterPrx;
typedef Ice_ObjectPrx* IDM_RouterPrxPtr;

/* forward declaration */
static void
IDM_Router_methodHandler(IDM_RouterPtr self,
                         Ice_InputStreamPtr is,
                         Ice_OutputStreamPtr os);

static void
IDM_Router_init(IDM_RouterPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)IDM_Router_methodHandler);
}

static void
IDM_Router_methodHandler(IDM_RouterPtr self,
                         Ice_InputStreamPtr is,
                         Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "adv") == 0) {
        IDM_NeighborDiscovery_Listener_adv__dispatch((IDM_NeighborDiscovery_ListenerPtr)self,
                is,
                os);
        return;
    }
    
    if (strcmp(operation, "bye") == 0) {
        IDM_NeighborDiscovery_Listener_bye__dispatch((IDM_NeighborDiscovery_ListenerPtr)self,
                is,
                os);
        return;
    }
    
    if (strcmp(operation, "flowAdd") == 0) {
        IDM_Routing_RouterAdmin_flowAdd__dispatch((IDM_Routing_RouterAdminPtr)self,
                is,
                os);
        return;
    }
    
    if (strcmp(operation, "flowDelete") == 0) {
        IDM_Routing_RouterAdmin_flowDelete__dispatch((IDM_Routing_RouterAdminPtr)self,
                is,
                os);
        return;
    }
    
    if (strcmp(operation, "flowList") == 0) {
        IDM_Routing_RouterAdmin_flowList__dispatch((IDM_Routing_RouterAdminPtr)self,
                is,
                os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

typedef struct {
    Ice_Object __base;
} IDM_Controller;

typedef IDM_Controller* IDM_ControllerPtr;

typedef Ice_ObjectPrx IDM_ControllerPrx;
typedef Ice_ObjectPrx* IDM_ControllerPrxPtr;

/* forward declaration */
static void
IDM_Controller_methodHandler(IDM_ControllerPtr self,
                             Ice_InputStreamPtr is,
                             Ice_OutputStreamPtr os);

static void
IDM_Controller_init(IDM_ControllerPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)IDM_Controller_methodHandler);
}

void
IDM_ControllerI_newNeighbor(IDM_ControllerPtr self,
                            Ice_String proxy,
                            Ice_String routerAddr) __attribute__((weak));

void
IDM_ControllerI_newRouter(IDM_ControllerPtr self,
                          Ice_String proxy) __attribute__((weak));

void
IDM_ControllerI_matchingError(IDM_ControllerPtr self,
                              IDM_Message m) __attribute__((weak));

static void
IDM_Controller_newNeighbor__dispatch(IDM_ControllerPtr self,
                                     Ice_InputStreamPtr is,
                                     Ice_OutputStreamPtr os) {
    Ice_String proxy;
    Ice_String routerAddr;

    trace();
    
    Ice_InputStream_readIceString(is, &proxy);
    
    Ice_InputStream_readIceString(is, &routerAddr);
    
    IDM_ControllerI_newNeighbor(self, proxy, routerAddr);
}

static void
IDM_Controller_newRouter__dispatch(IDM_ControllerPtr self,
                                   Ice_InputStreamPtr is,
                                   Ice_OutputStreamPtr os) {
    Ice_String proxy;

    trace();
    
    Ice_InputStream_readIceString(is, &proxy);
    
    IDM_ControllerI_newRouter(self, proxy);
}

static void
IDM_Controller_matchingError__dispatch(IDM_ControllerPtr self,
                                       Ice_InputStreamPtr is,
                                       Ice_OutputStreamPtr os) {
    IDM_Message m;

    trace();
    
    IDM_Message_readFromInputStream(&m, is);
    
    IDM_ControllerI_matchingError(self, m);
}

static void
IDM_Controller_methodHandler(IDM_ControllerPtr self,
                             Ice_InputStreamPtr is,
                             Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "newNeighbor") == 0) {
        IDM_Controller_newNeighbor__dispatch(self, is, os);
        return;
    }
    
    if (strcmp(operation, "newRouter") == 0) {
        IDM_Controller_newRouter__dispatch(self, is, os);
        return;
    }
    
    if (strcmp(operation, "matchingError") == 0) {
        IDM_Controller_matchingError__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
IDM_Controller_newNeighbor(IDM_ControllerPrxPtr self,
                           Ice_String proxy,
                           Ice_String routerAddr) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "newNeighbor");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeIceString(os, proxy);
    Ice_OutputStream_writeIceString(os, routerAddr);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

static void
IDM_Controller_newRouter(IDM_ControllerPrxPtr self,
                         Ice_String proxy) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "newRouter");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeIceString(os, proxy);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

static void
IDM_Controller_matchingError(IDM_ControllerPrxPtr self,
                             IDM_Message m) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "matchingError");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    IDM_Message_writeToOutputStream(&(m), os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Int size;
    Ice_Byte* items;
} DUO_IDM_Address;

typedef DUO_IDM_Address* DUO_IDM_AddressPtr;

static void
DUO_IDM_Address_writeToOutputStream(DUO_IDM_AddressPtr self,
                                    Ice_OutputStreamPtr os) {
    int i = 0;

    trace();
    
    Ptr_check(os);
    
    Ice_OutputStream_writeSize(os, self->size);
    
    for (i=0; i<self->size; i++) {
        Ice_OutputStream_writeByte(os, *(self->items + i));
    }
}

static void
DUO_IDM_Address_readFromInputStream(DUO_IDM_AddressPtr self,
                                    Ice_InputStreamPtr is) {

    trace();
    
    Ptr_check(is);
    
    Ice_InputStream_readSize(is, &(self->size), true);
    
    /* zero-copy optimization */
    self->items = is->next;
    is->next += self->size;
}

typedef struct {
    Ice_Object __base;
} DUO_IDM_IBool_W;

typedef DUO_IDM_IBool_W* DUO_IDM_IBool_WPtr;

typedef Ice_ObjectPrx DUO_IDM_IBool_WPrx;
typedef Ice_ObjectPrx* DUO_IDM_IBool_WPrxPtr;

/* forward declaration */
static void
DUO_IDM_IBool_W_methodHandler(DUO_IDM_IBool_WPtr self,
                              Ice_InputStreamPtr is,
                              Ice_OutputStreamPtr os);

static void
DUO_IDM_IBool_W_init(DUO_IDM_IBool_WPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)DUO_IDM_IBool_W_methodHandler);
}

void
DUO_IDM_IBool_WI_set(DUO_IDM_IBool_WPtr self,
                     Ice_Bool v,
                     DUO_IDM_Address source) __attribute__((weak));

static void
DUO_IDM_IBool_W_set__dispatch(DUO_IDM_IBool_WPtr self,
                              Ice_InputStreamPtr is,
                              Ice_OutputStreamPtr os) {
    Ice_Bool v = 0;
    DUO_IDM_Address source;

    trace();
    
    Ice_InputStream_readBool(is, &v);
    
    /* allocate space for sequence */
    Ice_InputStream_readSize(is, &source.size, false);
    byte _data[sizeof(byte) * source.size];
    source.items = (byte*)_data;
    DUO_IDM_Address_readFromInputStream(&source, is);
    
    DUO_IDM_IBool_WI_set(self, v, source);
}

static void
DUO_IDM_IBool_W_methodHandler(DUO_IDM_IBool_WPtr self,
                              Ice_InputStreamPtr is,
                              Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "set") == 0) {
        DUO_IDM_IBool_W_set__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
DUO_IDM_IBool_W_set(DUO_IDM_IBool_WPrxPtr self,
                    Ice_Bool v,
                    DUO_IDM_Address source) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "set");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeBool(os, v);
    DUO_IDM_Address_writeToOutputStream(&(source), os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Object __base;
} DUO_IDM_IByte_W;

typedef DUO_IDM_IByte_W* DUO_IDM_IByte_WPtr;

typedef Ice_ObjectPrx DUO_IDM_IByte_WPrx;
typedef Ice_ObjectPrx* DUO_IDM_IByte_WPrxPtr;

/* forward declaration */
static void
DUO_IDM_IByte_W_methodHandler(DUO_IDM_IByte_WPtr self,
                              Ice_InputStreamPtr is,
                              Ice_OutputStreamPtr os);

static void
DUO_IDM_IByte_W_init(DUO_IDM_IByte_WPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)DUO_IDM_IByte_W_methodHandler);
}

void
DUO_IDM_IByte_WI_set(DUO_IDM_IByte_WPtr self,
                     Ice_Byte v,
                     DUO_IDM_Address source) __attribute__((weak));

static void
DUO_IDM_IByte_W_set__dispatch(DUO_IDM_IByte_WPtr self,
                              Ice_InputStreamPtr is,
                              Ice_OutputStreamPtr os) {
    Ice_Byte v = 0;
    DUO_IDM_Address source;

    trace();
    
    Ice_InputStream_readByte(is, &v);
    
    /* allocate space for sequence */
    Ice_InputStream_readSize(is, &source.size, false);
    byte _data[sizeof(byte) * source.size];
    source.items = (byte*)_data;
    DUO_IDM_Address_readFromInputStream(&source, is);
    
    DUO_IDM_IByte_WI_set(self, v, source);
}

static void
DUO_IDM_IByte_W_methodHandler(DUO_IDM_IByte_WPtr self,
                              Ice_InputStreamPtr is,
                              Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "set") == 0) {
        DUO_IDM_IByte_W_set__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
DUO_IDM_IByte_W_set(DUO_IDM_IByte_WPrxPtr self,
                    Ice_Byte v,
                    DUO_IDM_Address source) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "set");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeByte(os, v);
    DUO_IDM_Address_writeToOutputStream(&(source), os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Object __base;
} DUO_IDM_IInt_W;

typedef DUO_IDM_IInt_W* DUO_IDM_IInt_WPtr;

typedef Ice_ObjectPrx DUO_IDM_IInt_WPrx;
typedef Ice_ObjectPrx* DUO_IDM_IInt_WPrxPtr;

/* forward declaration */
static void
DUO_IDM_IInt_W_methodHandler(DUO_IDM_IInt_WPtr self,
                             Ice_InputStreamPtr is,
                             Ice_OutputStreamPtr os);

static void
DUO_IDM_IInt_W_init(DUO_IDM_IInt_WPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)DUO_IDM_IInt_W_methodHandler);
}

void
DUO_IDM_IInt_WI_set(DUO_IDM_IInt_WPtr self,
                    Ice_Int v,
                    DUO_IDM_Address source) __attribute__((weak));

static void
DUO_IDM_IInt_W_set__dispatch(DUO_IDM_IInt_WPtr self,
                             Ice_InputStreamPtr is,
                             Ice_OutputStreamPtr os) {
    Ice_Int v = 0;
    DUO_IDM_Address source;

    trace();
    
    Ice_InputStream_readInt(is, &v);
    
    /* allocate space for sequence */
    Ice_InputStream_readSize(is, &source.size, false);
    byte _data[sizeof(byte) * source.size];
    source.items = (byte*)_data;
    DUO_IDM_Address_readFromInputStream(&source, is);
    
    DUO_IDM_IInt_WI_set(self, v, source);
}

static void
DUO_IDM_IInt_W_methodHandler(DUO_IDM_IInt_WPtr self,
                             Ice_InputStreamPtr is,
                             Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "set") == 0) {
        DUO_IDM_IInt_W_set__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
DUO_IDM_IInt_W_set(DUO_IDM_IInt_WPrxPtr self,
                   Ice_Int v,
                   DUO_IDM_Address source) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "set");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeInt(os, v);
    DUO_IDM_Address_writeToOutputStream(&(source), os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Object __base;
} DUO_IDM_IString_W;

typedef DUO_IDM_IString_W* DUO_IDM_IString_WPtr;

typedef Ice_ObjectPrx DUO_IDM_IString_WPrx;
typedef Ice_ObjectPrx* DUO_IDM_IString_WPrxPtr;

/* forward declaration */
static void
DUO_IDM_IString_W_methodHandler(DUO_IDM_IString_WPtr self,
                                Ice_InputStreamPtr is,
                                Ice_OutputStreamPtr os);

static void
DUO_IDM_IString_W_init(DUO_IDM_IString_WPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)DUO_IDM_IString_W_methodHandler);
}

void
DUO_IDM_IString_WI_set(DUO_IDM_IString_WPtr self,
                       Ice_String v,
                       DUO_IDM_Address source) __attribute__((weak));

static void
DUO_IDM_IString_W_set__dispatch(DUO_IDM_IString_WPtr self,
                                Ice_InputStreamPtr is,
                                Ice_OutputStreamPtr os) {
    Ice_String v;
    DUO_IDM_Address source;

    trace();
    
    Ice_InputStream_readIceString(is, &v);
    
    /* allocate space for sequence */
    Ice_InputStream_readSize(is, &source.size, false);
    byte _data[sizeof(byte) * source.size];
    source.items = (byte*)_data;
    DUO_IDM_Address_readFromInputStream(&source, is);
    
    DUO_IDM_IString_WI_set(self, v, source);
}

static void
DUO_IDM_IString_W_methodHandler(DUO_IDM_IString_WPtr self,
                                Ice_InputStreamPtr is,
                                Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "set") == 0) {
        DUO_IDM_IString_W_set__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
DUO_IDM_IString_W_set(DUO_IDM_IString_WPrxPtr self,
                      Ice_String v,
                      DUO_IDM_Address source) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "set");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeIceString(os, v);
    DUO_IDM_Address_writeToOutputStream(&(source), os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Object __base;
} DUO_IDM_Active_R;

typedef DUO_IDM_Active_R* DUO_IDM_Active_RPtr;

typedef Ice_ObjectPrx DUO_IDM_Active_RPrx;
typedef Ice_ObjectPrx* DUO_IDM_Active_RPrxPtr;

/* forward declaration */
static void
DUO_IDM_Active_R_methodHandler(DUO_IDM_Active_RPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os);

static void
DUO_IDM_Active_R_init(DUO_IDM_Active_RPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)DUO_IDM_Active_R_methodHandler);
}

DUO_IDM_Address
DUO_IDM_Active_RI_getObserver(DUO_IDM_Active_RPtr self) __attribute__((weak));

static void
DUO_IDM_Active_R_getObserver__dispatch(DUO_IDM_Active_RPtr self,
                                       Ice_InputStreamPtr is,
                                       Ice_OutputStreamPtr os) {
    DUO_IDM_Address _retval;

    trace();
    
    _retval = DUO_IDM_Active_RI_getObserver(self);
    DUO_IDM_Address_writeToOutputStream(&(_retval), os);
}

static void
DUO_IDM_Active_R_methodHandler(DUO_IDM_Active_RPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "getObserver") == 0) {
        DUO_IDM_Active_R_getObserver__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static DUO_IDM_Address
DUO_IDM_Active_R_getObserver(DUO_IDM_Active_RPrxPtr self) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "getObserver");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Object __base;
} DUO_IDM_Active_W;

typedef DUO_IDM_Active_W* DUO_IDM_Active_WPtr;

typedef Ice_ObjectPrx DUO_IDM_Active_WPrx;
typedef Ice_ObjectPrx* DUO_IDM_Active_WPrxPtr;

/* forward declaration */
static void
DUO_IDM_Active_W_methodHandler(DUO_IDM_Active_WPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os);

static void
DUO_IDM_Active_W_init(DUO_IDM_Active_WPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)DUO_IDM_Active_W_methodHandler);
}

void
DUO_IDM_Active_WI_setObserver(DUO_IDM_Active_WPtr self,
                              DUO_IDM_Address observer) __attribute__((weak));

static void
DUO_IDM_Active_W_setObserver__dispatch(DUO_IDM_Active_WPtr self,
                                       Ice_InputStreamPtr is,
                                       Ice_OutputStreamPtr os) {
    DUO_IDM_Address observer;

    trace();
    
    /* allocate space for sequence */
    Ice_InputStream_readSize(is, &observer.size, false);
    byte _data[sizeof(byte) * observer.size];
    observer.items = (byte*)_data;
    DUO_IDM_Address_readFromInputStream(&observer, is);
    
    DUO_IDM_Active_WI_setObserver(self, observer);
}

static void
DUO_IDM_Active_W_methodHandler(DUO_IDM_Active_WPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "setObserver") == 0) {
        DUO_IDM_Active_W_setObserver__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
DUO_IDM_Active_W_setObserver(DUO_IDM_Active_WPrxPtr self,
                             DUO_IDM_Address observer) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "setObserver");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    DUO_IDM_Address_writeToOutputStream(&(observer), os);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

typedef struct {
    Ice_Object __base;
} DUO_IDM_Listener;

typedef DUO_IDM_Listener* DUO_IDM_ListenerPtr;

typedef Ice_ObjectPrx DUO_IDM_ListenerPrx;
typedef Ice_ObjectPrx* DUO_IDM_ListenerPrxPtr;

/* forward declaration */
static void
DUO_IDM_Listener_methodHandler(DUO_IDM_ListenerPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os);

static void
DUO_IDM_Listener_init(DUO_IDM_ListenerPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)DUO_IDM_Listener_methodHandler);
}

void
DUO_IDM_ListenerI_adv(DUO_IDM_ListenerPtr self,
                      Ice_String address) __attribute__((weak));

void
DUO_IDM_ListenerI_bye(DUO_IDM_ListenerPtr self,
                      Ice_String address) __attribute__((weak));

static void
DUO_IDM_Listener_adv__dispatch(DUO_IDM_ListenerPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os) {
    Ice_String address;

    trace();
    
    Ice_InputStream_readIceString(is, &address);
    
    DUO_IDM_ListenerI_adv(self, address);
}

static void
DUO_IDM_Listener_bye__dispatch(DUO_IDM_ListenerPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os) {
    Ice_String address;

    trace();
    
    Ice_InputStream_readIceString(is, &address);
    
    DUO_IDM_ListenerI_bye(self, address);
}

static void
DUO_IDM_Listener_methodHandler(DUO_IDM_ListenerPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "adv") == 0) {
        DUO_IDM_Listener_adv__dispatch(self, is, os);
        return;
    }
    
    if (strcmp(operation, "bye") == 0) {
        DUO_IDM_Listener_bye__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
DUO_IDM_Listener_adv(DUO_IDM_ListenerPrxPtr self,
                     Ice_String address) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "adv");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeIceString(os, address);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

static void
DUO_IDM_Listener_bye(DUO_IDM_ListenerPrxPtr self,
                     Ice_String address) {
    Ice_OutputStreamPtr os = &(self->stream);

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    Ice_OutputStream_writeInt(os, 0);
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "bye");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeIceString(os, address);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* FIXME: wait response and return correct value */
    
    Ice_ObjectPrx_closeConnection(self);
}

